{------------------------------------------------------------------------------}
{                                                                              }
{  TWavePlayer v1.02                                                           }
{  by Kambiz R. Khojasteh                                                      }
{                                                                              }
{  kambiz@delphiarea.com                                                       }
{  http://www.delphiarea.com                                                   }
{                                                                              }
{------------------------------------------------------------------------------}

{$I DELPHIAREA.INC}

unit WPReg;

interface

uses
  Windows, Classes,
  {$IFDEF COMPILER6_UP} DesignIntf, DesignEditors {$ELSE} DsgnIntf {$ENDIF};

type

{ TWavePlayerEditor }

  TWavePlayerEditor = class(TComponentEditor)
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
    procedure Edit; override;
  end;

{ TWavePropertyEditor }

  TWavePropertyEditor = class(TStringProperty)
  public
    procedure Edit; override;
    function GetValue: string; override;
    procedure SetValue (const Value: string); override;
    function GetAttributes: TPropertyAttributes; override;
  end;

procedure Register;

implementation

uses
  WavePlay, Dialogs;

function LoadWavePlayerFromFile(WavePlayer: TWavePlayer): Boolean;
begin
  Result := False;
  with TOpenDialog.Create(nil) do
    try
      Filter := 'Wave files (*.wav;*.wave)|*.wav;*.wave|Any file (*.*)|*.*';
      Options := Options + [ofPathMustExist, ofFileMustExist];
      if Execute then
      begin
        WavePlayer.LoadFromFile(FileName);
        Result := True;
      end;
    finally
      Free;
    end
end;

{ TWavePlayerEditor }

procedure TWavePlayerEditor.ExecuteVerb(Index: Integer);
begin
  if Index = 0 then
  begin
    if LoadWavePlayerFromFile(Component as TWavePlayer) then
      Designer.Modified;
  end
  else if Index = 1 then
  begin
    (Component as TWavePlayer).Clear;
    Designer.Modified;
  end
  else
    inherited ExecuteVerb(Index);
end;

function TWavePlayerEditor.GetVerb(Index: Integer): string;
begin
  case Index of
    0: Result := 'Load Sound From File...';
    1: Result := 'Clear Sound';
  else
    Result := '';
  end;
end;

function TWavePlayerEditor.GetVerbCount: Integer;
begin
  Result := 2;
end;

procedure TWavePlayerEditor.Edit;
begin
  ExecuteVerb(0);
end;

{ TWavePropertyEditor }

function TWavePropertyEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog];
end;

function TWavePropertyEditor.GetValue: string;
begin
  if TWave(GetOrdValue).Size > 0 then
    Result := '(TWave)'
  else
    Result := '(None)';
end;

procedure TWavePropertyEditor.SetValue(const Value: string);
begin
  if Value = '' then SetOrdValue(0);
end;

procedure TWavePropertyEditor.Edit;
begin
  if LoadWavePlayerFromFile(GetComponent(0) as TWavePlayer) then
    Modified;
end;

procedure Register;
begin
  RegisterComponents('Delphi Area', [TWavePlayer]);
  RegisterComponentEditor(TWavePlayer, TWavePlayerEditor);
  RegisterPropertyEditor(TypeInfo(TWave), TWavePlayer, 'Sound', TWavePropertyEditor);
end;

end.
